/* Analyse a pcap file and extract the packets inter arrival time 

	contact: <gregor@majordomus.org>

Copyright (c) 2005 Gregor Maier.  All rights reserved.


Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the names of the copyright owners nor the names of its
   contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.
4. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
       This product includes software developed by Anzen Computing, Inc.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pcap.h>


#define tv_to_double(tv) ((tv).tv_sec + 1e-6*(tv).tv_usec) 

int main (int argc, char* argv[]) 
{
	char optstring[] = "r:hv";
	int opt;
	char *filename=NULL;
	pcap_t *pd;
	char errbuf[PCAP_ERRBUF_SIZE+1];
	struct pcap_pkthdr hdr;
	struct timeval firstpkt, lastpkt;
	struct timeval tv_offset, tv_iat;
	double d_offset, d_iat;
	int is_first=1;
	
	
	while ((opt=getopt(argc, argv, optstring))!=-1) {
		switch (opt) {
			case 'h':
				printf("get_iat - Read a pcap dump file and extract inter arrival times\n\n");
				printf("Usage: get_iat [-r pcapfile] [-h]\n");
				printf("    -h             Print this message.\n");
				printf("    -r pcapfile    Read pcapfile if omitted stdin is used.\n");
				printf("\nOutput:\n");
				printf("Two tab seperated columns with floats.\nOne line for each packet (except the first packet)\n");
				printf("First Column:  Arrival time difference relative to first packet\n");
				printf("Second Column:  Arrival time difference relative to previous packet\n");
				printf("\n");
				exit(0);
				break;
			case 'v':
				printf("This should print the version number\n");
				exit(0);
				break;
			case 'r':
				if ((filename=strdup(optarg))==NULL) {
					fprintf(stderr, "%s: ERROR: Could not allocate memory\n", argv[0]);
					exit(1);
				}
				break;
			default:
				fprintf(stderr, "%s: getopt returned an unknown character code %x\n", argv[0], opt);
				break;
		}
	}
	if (filename==NULL) {
		/* XXX: Mayber error detection here ... */
		filename=strdup("-");
	}
	pd = pcap_open_offline(filename, errbuf);
	if (pd==NULL) {
		fprintf(stderr, "%s: pcap_open_offline: %s\n", argv[0], errbuf);
		exit(1);
	}
	
	while (pcap_next(pd, &hdr)) {
		if (is_first) {
			is_first=0;
			firstpkt = lastpkt = hdr.ts;
		}
		else {
			d_offset = tv_to_double(hdr.ts)-tv_to_double(firstpkt);
			d_iat = tv_to_double(hdr.ts) - tv_to_double(lastpkt);
			fprintf(stdout, "%.8lf\t%.8lf\n", d_offset, d_iat);
			lastpkt = hdr.ts;
		}
	}

	pcap_close(pd);
	if (filename!=NULL)
		free(filename);
	exit(0);
}
